# Fields

An object editor for Dart.  Also known as an 'object inspector' or a 
'property grid'.

[![Build Status](https://drone.io/github.com/rmsmith/fields/status.png)][badge]

![object editor][object editor dark]

## Status (Dart SDK r26297)

The examples work in Dartium.

The examples fail compilation with `dart2js` due to [issue 12287][].

Issues that you can star to improve this project are:

- [issue 6433][]: InstanceMirror get reflectedType;
- ~~[issue 10682][]: InstanceMirror : request for `Stream onFieldChanged` 
member~~ __NOT PLANNED :(__
- [issue 11186][]: Add `typedef bool Predicate<E>(E element)` to dart:core
- ~~[issue 11345][]: Generic type parameter as literal in the VM.~~ __FIXED in
r24861__
- ~~[issue 11550][]: Metadata call to constant constructor regression in
 dart2js.~~ __FIXED in r24595__
- [issue 12022][]: dart2js: VariableMirror.type always returns `dynamic`
- [issue 12287][]: dart2js: type parameter as literal in constant expression
- ~~[issue 12319][]: vm: getting the `members` of a ClassMirror on 'Null' 
crash~~ __FIXED in r25982__

## Documentation

Please consult the [API documentation][api].

## Use

### Create an ObjectEditor

```dart
import 'dart:html';
import 'package:fields/fields.dart';

main() {  
  var editor = new ObjectEditor();
  // Append the editor's element to your document where and when you like.
  document.body.append(editor.element);
}
```

### Select an Object

```dart
...
// Select any dart object instance for edition.
editor.selected = new KitchenSink();
...
```

### Annotate Fields

By default, we do our best to provide a standard editor for each of your 
object's fields based on its `Type`.  However, sometimes you want to tell us
more about your fields and you may do this by applying annotations from the
`fields.meta` library:

```dart
import 'package:fields/fields.dart';

// Annotate your model objects to constrain the editor for any field.
class Sunflower {
  @UseFieldEditor(ColorEditor)
  String color = "#ff8000";
  @UseFieldEditor(RangeEditor) 
  @UseRange(min: 0, max: 1000)  
  int seeds = 500;
  @UseFieldEditor(NumberEditor)
  @UseRange(min: 1, max:3) 
  int seedRadius = 2;
}
```

## Extend

TODO:

## Theme

![light][object editor light]![dark][object editor dark]

_Fields uses the MIT license as described in the [LICENSE][license] file, and 
follows [semantic versioning][]._

_The default icons were created by [glyphicons.com][] and are released under the 
[Creative Commons Attribution 3.0 Unported][creative commons]._

[api]: http://rmsmith.github.com/fields/
[badge]: https://drone.io/github.com/rmsmith/fields/latest
[creative commons]: http://creativecommons.org/licenses/by/3.0/deed.en
[glyphicons.com]: http://glyphicons.com/
[issue 6433]: https://code.google.com/p/dart/issues/detail?id=6433
[issue 10682]: https://code.google.com/p/dart/issues/detail?id=10682
[issue 11186]: https://code.google.com/p/dart/issues/detail?id=11186
[issue 11345]: https://code.google.com/p/dart/issues/detail?id=11345
[issue 11550]: https://code.google.com/p/dart/issues/detail?id=11550
[issue 12022]: https://code.google.com/p/dart/issues/detail?id=12022
[issue 12287]: https://code.google.com/p/dart/issues/detail?id=12287
[issue 12319]: https://code.google.com/p/dart/issues/detail?id=12319
[license]: https://github.com/rmsmith/fields/blob/master/LICENSE
[object editor light]: https://github.com/rmsmith/fields/blob/master/doc/res/object_editor.png?raw=true
[object editor dark]: https://github.com/rmsmith/fields/blob/master/doc/res/object_editor_dark.png?raw=true
[semantic versioning]: http://semver.org/
