library fields.mirrors;

import 'dart:mirrors';

// TODO(rms): https://code.google.com/p/dart/issues/detail?id=6433
Type typeOf(TypeMirror tm) {
  if (tm.simpleName == const Symbol('bool'))          return bool;
  else if (tm.simpleName == const Symbol('DateTime')) return DateTime;
  else if (tm.simpleName == const Symbol('double'))   return double;
  else if (tm.simpleName == const Symbol('Duration')) return Duration;  
  else if (tm.simpleName == const Symbol('dynamic'))  return Dynamic;
  else if (tm.simpleName == const Symbol('int'))      return int;
  else if (tm.simpleName == const Symbol('List'))     return List;
  else if (tm.simpleName == const Symbol('Map'))      return Map;
  else if (tm.simpleName == const Symbol('num'))      return num;
  else if (tm.simpleName == const Symbol('String'))   return String;
  else                                                return null;
}

class _MakerOf<T> {
  const _MakerOf();
  get type => T;
}

final Type Dynamic = const _MakerOf().type;

bool isSimpleType(Type t) {
  switch(t) {
    case bool:
    case double:
    case int:    
    case List:
    case Map:
    case Object:
    case num:
    case String:
      return true;
    default:
      return Dynamic == t;
  }
}

final _objectClassMirror = reflectClass(Object);

/// Returns `true` of the given [type] and the given [other] represent the same
/// type, or if the [type] is in the inheritance hierarchy of [other], or if
/// [other] implements the interface of [type]; else returns `false`.
bool isAssignableFrom(ClassMirror type, ClassMirror other) {
  assert(type != null);
  assert(other != null);
  if (other == type) return true;
  if (other.superclass == null) return type == _objectClassMirror;
  if (other.superclass == type) return true;
  if (other.superinterfaces.any((i) => i == type)) return true;
  return isAssignableFrom(type, other.superclass);
}
